@@ -6,7 +6,7 @@
 
 /*
 Author:
-	Artur Agostinho Arajo <b5498@math.uminho.pt>
+	Artur Agostinho Araujo <artur.stat@gmail.com>
 
 Description:
 	Computes quantiles of a given vector.
