@@ -3,7 +3,7 @@
 
 /*
 Author:
-	Artur Agostinho Arajo <b5498@math.uminho.pt>
+	Artur Agostinho Araujo <artur.stat@gmail.com>
 
 Description:
 	Computes unique index vector.
